@@ -18,11 +18,12 @@
  *   for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *   along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
  *          [http://www.gnu.org/licenses/gpl-2.0.html]
  *
+ *   SPDX-License-Identifier: GPL-2.0+
+ *
  */
 
 #include <stdio.h>
@@ -978,7 +979,7 @@ top:
 			if (ctx->cp900)
 				state = S_PRINTER_CP900_FOOTER;
 			else
-				state = S_FINISHED;
+				state = S_PRINTER_DONE;
 		}
 		break;
 	case S_PRINTER_CP900_FOOTER: {
@@ -1051,7 +1052,7 @@ static void canonselphy_cmdline(void)
 
 struct dyesub_backend canonselphy_backend = {
 	.name = "Canon SELPHY CP/ES",
-	.version = "0.93",
+	.version = "0.94",
 	.uri_prefix = "canonselphy",
 	.cmdline_usage = canonselphy_cmdline,
 	.cmdline_arg = canonselphy_cmdline_arg,
@@ -1061,37 +1062,37 @@ struct dyesub_backend canonselphy_backend = {
 	.read_parse = canonselphy_read_parse,
 	.main_loop = canonselphy_main_loop,
 	.devices = {
-	{ USB_VID_CANON, USB_PID_CANON_CP10, P_CP10, ""},
-	{ USB_VID_CANON, USB_PID_CANON_CP100, P_CP_XXX, ""},
-	{ USB_VID_CANON, USB_PID_CANON_CP200, P_CP_XXX, ""},
-	{ USB_VID_CANON, USB_PID_CANON_CP220, P_CP_XXX, ""},
-	{ USB_VID_CANON, USB_PID_CANON_CP300, P_CP_XXX, ""},
-	{ USB_VID_CANON, USB_PID_CANON_CP330, P_CP_XXX, ""},
-	{ USB_VID_CANON, USB_PID_CANON_CP400, P_CP_XXX, ""},
-	{ USB_VID_CANON, USB_PID_CANON_CP500, P_CP_XXX, ""},
-	{ USB_VID_CANON, USB_PID_CANON_CP510, P_CP_XXX, ""},
-	{ USB_VID_CANON, USB_PID_CANON_CP520, P_CP_XXX, ""},
-	{ USB_VID_CANON, USB_PID_CANON_CP530, P_CP_XXX, ""},
-	{ USB_VID_CANON, USB_PID_CANON_CP600, P_CP_XXX, ""},
-	{ USB_VID_CANON, USB_PID_CANON_CP710, P_CP_XXX, ""},
-	{ USB_VID_CANON, USB_PID_CANON_CP720, P_CP_XXX, ""},
-	{ USB_VID_CANON, USB_PID_CANON_CP730, P_CP_XXX, ""},
-	{ USB_VID_CANON, USB_PID_CANON_CP740, P_CP_XXX, ""},
-	{ USB_VID_CANON, USB_PID_CANON_CP750, P_CP_XXX, ""},
-	{ USB_VID_CANON, USB_PID_CANON_CP760, P_CP_XXX, ""},
-	{ USB_VID_CANON, USB_PID_CANON_CP770, P_CP_XXX, ""},
-	{ USB_VID_CANON, USB_PID_CANON_CP780, P_CP_XXX, ""},
-	{ USB_VID_CANON, USB_PID_CANON_CP790, P_CP790, ""},
-	{ USB_VID_CANON, USB_PID_CANON_CP800, P_CP_XXX, ""},
-	{ USB_VID_CANON, USB_PID_CANON_CP810, P_CP_XXX, ""},
-	{ USB_VID_CANON, USB_PID_CANON_CP900, P_CP_XXX, ""},
-	{ USB_VID_CANON, USB_PID_CANON_ES1, P_ES1, ""},
-	{ USB_VID_CANON, USB_PID_CANON_ES2, P_ES2_20, ""},
-	{ USB_VID_CANON, USB_PID_CANON_ES20, P_ES2_20, ""},
-	{ USB_VID_CANON, USB_PID_CANON_ES3, P_ES3_30, ""},
-	{ USB_VID_CANON, USB_PID_CANON_ES30, P_ES3_30, ""},
-	{ USB_VID_CANON, USB_PID_CANON_ES40, P_ES40, ""},
-	{ 0, 0, 0, ""}
+	{ USB_VID_CANON, USB_PID_CANON_CP10, P_CP10, NULL},
+	{ USB_VID_CANON, USB_PID_CANON_CP100, P_CP_XXX, NULL},
+	{ USB_VID_CANON, USB_PID_CANON_CP200, P_CP_XXX, NULL},
+	{ USB_VID_CANON, USB_PID_CANON_CP220, P_CP_XXX, NULL},
+	{ USB_VID_CANON, USB_PID_CANON_CP300, P_CP_XXX, NULL},
+	{ USB_VID_CANON, USB_PID_CANON_CP330, P_CP_XXX, NULL},
+	{ USB_VID_CANON, USB_PID_CANON_CP400, P_CP_XXX, NULL},
+	{ USB_VID_CANON, USB_PID_CANON_CP500, P_CP_XXX, NULL},
+	{ USB_VID_CANON, USB_PID_CANON_CP510, P_CP_XXX, NULL},
+	{ USB_VID_CANON, USB_PID_CANON_CP520, P_CP_XXX, NULL},
+	{ USB_VID_CANON, USB_PID_CANON_CP530, P_CP_XXX, NULL},
+	{ USB_VID_CANON, USB_PID_CANON_CP600, P_CP_XXX, NULL},
+	{ USB_VID_CANON, USB_PID_CANON_CP710, P_CP_XXX, NULL},
+	{ USB_VID_CANON, USB_PID_CANON_CP720, P_CP_XXX, NULL},
+	{ USB_VID_CANON, USB_PID_CANON_CP730, P_CP_XXX, NULL},
+	{ USB_VID_CANON, USB_PID_CANON_CP740, P_CP_XXX, NULL},
+	{ USB_VID_CANON, USB_PID_CANON_CP750, P_CP_XXX, NULL},
+	{ USB_VID_CANON, USB_PID_CANON_CP760, P_CP_XXX, NULL},
+	{ USB_VID_CANON, USB_PID_CANON_CP770, P_CP_XXX, NULL},
+	{ USB_VID_CANON, USB_PID_CANON_CP780, P_CP_XXX, NULL},
+	{ USB_VID_CANON, USB_PID_CANON_CP790, P_CP790, NULL},
+	{ USB_VID_CANON, USB_PID_CANON_CP800, P_CP_XXX, NULL},
+	{ USB_VID_CANON, USB_PID_CANON_CP810, P_CP_XXX, NULL},
+	{ USB_VID_CANON, USB_PID_CANON_CP900, P_CP_XXX, NULL},
+	{ USB_VID_CANON, USB_PID_CANON_ES1, P_ES1, NULL},
+	{ USB_VID_CANON, USB_PID_CANON_ES2, P_ES2_20, NULL},
+	{ USB_VID_CANON, USB_PID_CANON_ES20, P_ES2_20, NULL},
+	{ USB_VID_CANON, USB_PID_CANON_ES3, P_ES3_30, NULL},
+	{ USB_VID_CANON, USB_PID_CANON_ES30, P_ES3_30, NULL},
+	{ USB_VID_CANON, USB_PID_CANON_ES40, P_ES40, NULL},
+	{ 0, 0, 0, NULL}
 	}
 };
 /*
