@@ -1,7 +1,7 @@
 /*
  *   DNP DS40/DS80 Photo Printer CUPS backend -- libusb-1.0 version
  *
- *   (c) 2013-2016 Solomon Peachy <pizza@shaftnet.org>
+ *   (c) 2013-2018 Solomon Peachy <pizza@shaftnet.org>
  *
  *   Development of this backend was sponsored by:
  *
@@ -24,11 +24,12 @@
  *   for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *   along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
  *          [http://www.gnu.org/licenses/gpl-2.0.html]
  *
+ *   SPDX-License-Identifier: GPL-2.0+
+ *
  */
 
 //#define DNP_ONLY
@@ -56,10 +57,12 @@
 #define USB_PID_DNP_DS40  0x0003 // Also Citizen CX
 #define USB_PID_DNP_DS80  0x0004 // Also Citizen CX-W, and Mitsubishi CP-3800DW
 #define USB_PID_DNP_DSRX1 0x0005 // Also Citizen CY
-#define USB_PID_CITIZEN_CW02 0x0006
 #define USB_PID_DNP_DS80D 0x0007
 #define USB_PID_DNP_DS620_OLD 0x0008
 
+#define USB_PID_CITIZEN_CW02 0x0006 // Also OP900II
+#define USB_PID_CITIZEN_CX02 0x000A
+
 #define USB_VID_DNP       0x1452
 #define USB_PID_DNP_DS620 0x8b01
 #define USB_PID_DNP_DS820 0x9001
@@ -98,6 +101,7 @@ struct dnpds40_ctx {
 	int mediaoffset;
 	int manual_copies;
 	int correct_count;
+	int needs_mlot;
 
 	uint32_t native_width;
 	int supports_6x9;
@@ -238,6 +242,7 @@ static char *dnpds40_printer_type(int type)
 	case P_DNP_DSRX1: return "DSRX1";
 	case P_DNP_DS620: return "DS620";
 	case P_DNP_DS820: return "DS820";
+	case P_CITIZEN_OP900II: return "OP900ii";
 	default: break;
 	}
 	return "Unknown";
@@ -652,6 +657,10 @@ static void dnpds40_attach(void *vctx, struct libusb_device_handle *dev,
 			ctx->supports_matte = 1;
 		if (FW_VER_CHECK(1,40))
 			ctx->supports_2x6 = 1;
+		if (FW_VER_CHECK(1,50))
+			ctx->supports_3x5x2 = 1;
+		if (FW_VER_CHECK(1,60))
+			ctx->supports_fullcut = ctx->supports_6x6 = 1; // No 5x5!
 		break;
 	case P_DNP_DS80:
 	case P_DNP_DS80D:
@@ -670,9 +679,22 @@ static void dnpds40_attach(void *vctx, struct libusb_device_handle *dev,
                 if (FW_VER_CHECK(1,20))
 			ctx->supports_3x5x2 = 1;
 		if (FW_VER_CHECK(2,00)) { /* AKA RX1HS */
+			ctx->needs_mlot = 1;
 			ctx->supports_mediaoffset = 1;
 			ctx->supports_iserial = 1;
 		}
+		if (FW_VER_CHECK(2,06)) {
+			ctx->supports_5x5 = ctx->supports_6x6 = 1;
+		}
+		break;
+	case P_CITIZEN_OP900II:
+		ctx->native_width = 1920;
+		ctx->supports_counterp = 1;
+		ctx->supports_matte = 1;
+		ctx->supports_mqty_default = 1;
+		ctx->supports_6x9 = 1;
+		if (FW_VER_CHECK(1,11))
+			ctx->supports_2x6 = 1;
 		break;
 	case P_DNP_DS620:
 		ctx->native_width = 1920;
@@ -698,9 +720,11 @@ static void dnpds40_attach(void *vctx, struct libusb_device_handle *dev,
 		if (FW_VER_CHECK(1,20))
 			ctx->supports_adv_fullcut = ctx->supports_advmatte = 1;
 		if (FW_VER_CHECK(1,30))
-			ctx->supports_luster = ctx->supports_finematte = 1;
+			ctx->supports_luster = 1;
 		if (FW_VER_CHECK(1,33))
 			ctx->supports_media_ext = 1;
+		if (FW_VER_CHECK(1,52))
+			ctx->supports_finematte = 1;
 		break;
 	case P_DNP_DS820:
 		ctx->native_width = 2560;
@@ -803,6 +827,9 @@ static void dnpds40_attach(void *vctx, struct libusb_device_handle *dev,
 			break;
 		case P_DNP_DSRX1:
 			switch (ctx->media) {
+			case 210: // 2L
+				ctx->media_count_new = 350;
+				break;
 			case 300: // PC
 				ctx->media_count_new = 700;
 				break;
@@ -814,6 +841,25 @@ static void dnpds40_attach(void *vctx, struct libusb_device_handle *dev,
 				break;
 			}
 			break;
+		case P_CITIZEN_OP900II:
+			switch (ctx->media) {
+			case 210: // 2L
+				ctx->media_count_new = 350;
+				break;
+			case 300: // PC
+				ctx->media_count_new = 600;
+				break;
+			case 310: // A5
+				ctx->media_count_new = 300;
+				break;
+			case 400: // A5W
+				ctx->media_count_new = 280;
+				break;
+			default:
+				ctx->media_count_new = 999; // non-zero
+				break;
+			}
+			break;
 		case P_DNP_DS80:
 		case P_DNP_DS80D:
 			switch (ctx->media) {
@@ -893,7 +939,7 @@ static int dnpds40_read_parse(void *vctx, int data_fd) {
 	ctx->databuf = malloc(MAX_PRINTJOB_LEN);
 	if (!ctx->databuf) {
 		ERROR("Memory allocation failure!\n");
-		return CUPS_BACKEND_CANCEL;
+		return CUPS_BACKEND_RETRY_CURRENT;
 	}
 
 	/* Clear everything out */
@@ -1373,6 +1419,26 @@ static int dnpds40_main_loop(void *vctx, int copies) {
 		ATTR("marker-types=ribbonWax\n");
 	}
 
+	/* RX1HS requires HS media, but the only way to tell is that the
+	   HS media reports a lot code, while the non-HS media does not. */
+	if (ctx->needs_mlot) {
+		/* Get Media Lot */
+		dnpds40_build_cmd(&cmd, "INFO", "MLOT", 0);
+
+		resp = dnpds40_resp_cmd(ctx, &cmd, &len);
+		if (!resp)
+			return CUPS_BACKEND_FAILED;
+
+		dnpds40_cleanup_string((char*)resp, len);
+
+		len = strlen((char*)resp);
+		free(resp);
+		if (!len) {
+			ERROR("Media does not report a valid lot number (non-HS media in RX1HS?)\n");
+			return CUPS_BACKEND_STOP;
+		}
+	}
+
 top:
 
 	/* Query status */
@@ -1566,7 +1632,7 @@ top:
 	}
 	sleep(1);  /* Give things a moment */
 
-	if (fast_return) {
+	if (fast_return && !ctx->manual_copies) {
 		INFO("Fast return mode enabled.\n");
 	} else {
 		INFO("Waiting for job to complete...\n");
@@ -2483,8 +2549,8 @@ static int dnpds40_cmdline_arg(void *vctx, int argc, char **argv)
 
 /* Exported */
 struct dyesub_backend dnpds40_backend = {
-	.name = "DNP DS40/DS80/DSRX1/DS620",
-	.version = "0.91",
+	.name = "DNP DS40/DS80/DSRX1/DS620/DS820",
+	.version = "0.97",
 	.uri_prefix = "dnpds40",
 	.cmdline_usage = dnpds40_cmdline,
 	.cmdline_arg = dnpds40_cmdline_arg,
@@ -2495,14 +2561,15 @@ struct dyesub_backend dnpds40_backend = {
 	.main_loop = dnpds40_main_loop,
 	.query_serno = dnpds40_query_serno,
 	.devices = {
-	{ USB_VID_CITIZEN, USB_PID_DNP_DS40, P_DNP_DS40, ""},
-	{ USB_VID_CITIZEN, USB_PID_DNP_DS80, P_DNP_DS80, ""},
-	{ USB_VID_CITIZEN, USB_PID_DNP_DSRX1, P_DNP_DSRX1, ""},
-	{ USB_VID_CITIZEN, USB_PID_DNP_DS620_OLD, P_DNP_DS620, ""},
-	{ USB_VID_DNP, USB_PID_DNP_DS620, P_DNP_DS620, ""},
-	{ USB_VID_DNP, USB_PID_DNP_DS80D, P_DNP_DS80D, ""},
-	{ USB_VID_CITIZEN, USB_PID_CITIZEN_CW02, P_DNP_DS40, ""},
-	{ USB_VID_DNP, USB_PID_DNP_DS820, P_DNP_DS820, ""},
-	{ 0, 0, 0, ""}
+	{ USB_VID_CITIZEN, USB_PID_DNP_DS40, P_DNP_DS40, NULL},
+	{ USB_VID_CITIZEN, USB_PID_DNP_DS80, P_DNP_DS80, NULL},
+	{ USB_VID_CITIZEN, USB_PID_DNP_DSRX1, P_DNP_DSRX1, NULL},
+	{ USB_VID_CITIZEN, USB_PID_DNP_DS620_OLD, P_DNP_DS620, NULL},
+	{ USB_VID_DNP, USB_PID_DNP_DS620, P_DNP_DS620, NULL},
+	{ USB_VID_DNP, USB_PID_DNP_DS80D, P_DNP_DS80D, NULL},
+	{ USB_VID_CITIZEN, USB_PID_CITIZEN_CW02, P_CITIZEN_OP900II, NULL},
+	{ USB_VID_CITIZEN, USB_PID_CITIZEN_CX02, P_DNP_DS620, NULL},
+	{ USB_VID_DNP, USB_PID_DNP_DS820, P_DNP_DS820, NULL},
+	{ 0, 0, 0, NULL}
 	}
 };
