@@ -18,11 +18,12 @@
  *   for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *   along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
  *          [http://www.gnu.org/licenses/gpl-2.0.html]
  *
+ *   SPDX-License-Identifier: GPL-2.0+
+ *
  */
 
 #include <stdio.h>
@@ -608,6 +609,9 @@ top:
 #define USB_VID_KODAK       0x040A
 #define USB_PID_KODAK_1400  0x4022
 #define USB_PID_KODAK_805   0x4034
+#define USB_VID_MITSU        0x06D3
+#define USB_PID_MITSU_3020D  0x038B
+#define USB_PID_MITSU_3020DA 0x03AA
 
 struct dyesub_backend kodak1400_backend = {
 	.name = "Kodak 1400/805",
@@ -623,7 +627,9 @@ struct dyesub_backend kodak1400_backend = {
 	.devices = {
 	{ USB_VID_KODAK, USB_PID_KODAK_1400, P_KODAK_1400_805, "Kodak"},
 	{ USB_VID_KODAK, USB_PID_KODAK_805, P_KODAK_1400_805, "Kodak"},
-	{ 0, 0, 0, ""}
+	{ USB_VID_MITSU, USB_PID_MITSU_3020D, P_KODAK_1400_805, NULL},
+	{ USB_VID_MITSU, USB_PID_MITSU_3020DA, P_KODAK_1400_805, NULL},
+	{ 0, 0, 0, NULL}
 	}
 };
 
