@@ -18,11 +18,12 @@
  *   for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *   along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
  *          [http://www.gnu.org/licenses/gpl-2.0.html]
  *
+ *   SPDX-License-Identifier: GPL-2.0+
+ *
  */
 
 #include <stdio.h>
@@ -115,8 +116,8 @@ typedef int (*send_image_dataFN)(struct BandImage *out, void *context,
 #define USB_PID_MITSU_D90   0x3B60
 #define USB_VID_KODAK       0x040a
 #define USB_PID_KODAK305    0x404f
-//#define USB_VID_FUJIFILM    XXXXXX
-//#define USB_PID_FUJI_ASK300 XXXXXX
+#define USB_VID_FUJIFILM    0x04cb
+#define USB_PID_FUJI_ASK300 0x5006
 
 /* Width of the laminate data file */
 #define LAMINATE_STRIDE 1864
@@ -304,7 +305,7 @@ struct mitsu70x_jobs {
 
 struct mitsu70x_status_deck {
 	uint8_t  mecha_status[2];
-	uint8_t  temperature;   /* D70 family only, K60 no */
+	uint8_t  temperature;   /* D70/D80 family only, K60 no? */
 	uint8_t  error_status[3];
 	uint8_t  rsvd_a[10];    /* K60 family [1] == temperature? [3:6] == lifetime prints in BCD */
 
@@ -328,21 +329,18 @@ struct mitsu70x_printerstatus_resp {
 	uint8_t  memory;
 	uint8_t  power;
 	uint8_t  unk[20];
-	uint8_t  sleeptime; /* In minutes, 0-10 */
+	uint8_t  sleeptime; /* In minutes, 0-60 */
 	uint8_t  iserial; /* 0x00 for Enabled, 0x80 for Disabled */
 	uint8_t  unk_b[12];
 	int16_t  model[6]; /* LE, UTF-16 */
 	int16_t  serno[6]; /* LE, UTF-16 */
 	struct mitsu70x_status_ver vers[7]; // components are 'MLRTF'
 	uint8_t  null[2];
-	uint8_t  user_serno[6];  /* Supposedly, don't know how to set it */
+	uint8_t  user_serno[6];  /* XXX Supposedly, don't know how to set it */
 	struct mitsu70x_status_deck lower;
 	struct mitsu70x_status_deck upper;
 } __attribute__((packed));
 
-#define EK305_0104_M_CSUM  0x2878  /* 1.04 316F8 3 2878 */
-#define MD70X_0112_M_CSUM  0x9FC3  /* 1.12 316W1 1 9FC3 */
-
 struct mitsu70x_memorystatus_resp {
 	uint8_t  hdr[3]; /* E4 56 33 */
 	uint8_t  memory;
@@ -355,7 +353,7 @@ struct mitsu70x_memorystatus_resp {
 struct mitsu70x_hdr {
 	uint8_t  hdr[4]; /* 1b 5a 54 XX */  // XXX also, seen 1b 5a 43!
 	uint16_t jobid;
-	uint8_t  rewind[2];  /* XXX K60/EK305/D80 only, 0 normally, 1 for "skip" ??? */
+	uint8_t  rewind[2];  /* K60/EK305/D80 only */
 	uint8_t  zero0[8];
 
 	uint16_t cols;
@@ -365,14 +363,14 @@ struct mitsu70x_hdr {
 	uint8_t  speed;
 	uint8_t  zero1[7];
 
-	uint8_t  deck; /* 0 = default, 1 = lower, 2 = upper */
+	uint8_t  deck; /* 0 = default, 1 = lower, 2 = upper -- Non-D70/D707 is always '1' */
 	uint8_t  zero2[7];
 	uint8_t  laminate; /* 00 == on, 01 == off */
 	uint8_t  laminate_mode; /* 00 == glossy, 02 == matte */
 	uint8_t  zero3[6];
 
 	uint8_t  multicut;
-	uint8_t  zero4[12];
+	uint8_t  zero4[12]; /* NOTE:  everything past this point is an extension */
 	uint8_t  sharpen;  /* 0-9.  5 is "normal", 0 is "off" */
 	uint8_t  mode;     /* 0 for cooked YMC planar, 1 for packed BGR */
 	uint8_t  use_lut;  /* in BGR mode, 0 disables, 1 enables */
@@ -380,8 +378,25 @@ struct mitsu70x_hdr {
 	uint8_t  pad[447];
 } __attribute__((packed));
 
+static int mitsu70x_get_printerstatus(struct mitsu70x_ctx *ctx, struct mitsu70x_printerstatus_resp *resp);
+
 /* Error dumps, etc */
 
+static char *mitsu70x_temperatures(uint8_t temp)
+{
+	switch(temp) {
+	case TEMPERATURE_NORMAL:
+		return "Normal";
+	case TEMPERATURE_PREHEAT:
+		return "Warming Up";
+	case TEMPERATURE_COOLING:
+		return "Cooling Down";
+	default:
+		break;
+	}
+	return "Unknown Temperature Status";
+}
+
 static char *mitsu70x_mechastatus(uint8_t *sts)
 {
 	switch(sts[0]) {
@@ -444,6 +459,15 @@ static char *mitsu70x_jobstatuses(uint8_t *sts)
 		case JOB_STATUS1_END_OK:
 			return "Normal End";
 		case JOB_STATUS1_END_HEADER:
+			switch(sts[2]) {
+			case JOB_STATUS2_END_HEADER_ERROR:
+				return "Incorrect Header data (bad print size?)";
+			case JOB_STATUS2_END_HEADER_MEMORY:
+				return "Insufficient printer memory";
+			default:
+				return "Unknown 'End Header' status2";
+			}
+			break;
 		case JOB_STATUS1_END_PRINT:
 			switch(sts[2]) {
 			case JOB_STATUS2_END_PRINT_MEDIA:
@@ -619,7 +643,7 @@ static const char *mitsu70x_media_types(uint8_t brand, uint8_t type)
 	else if (brand == 0xff && type == 0x05)
 		return "CK-D769 (6x9)";
 	else if (brand == 0xff && type == 0x0f)
-		return "CK-D768 (6x8)";
+		return "CK-D768/CK-D868 (6x8)";
 	else if (brand == 0x6c && type == 0x84)
 		return "Kodak 5R (5x7)";
 	else if (brand == 0x6c && type == 0x8f)
@@ -628,14 +652,21 @@ static const char *mitsu70x_media_types(uint8_t brand, uint8_t type)
 		return "CK-K57R (5x7)";
 	else if (brand == 0x61 && type == 0x8f)
 		return "CK-K76R (6x8)";
+	else if (brand == 0x7a && type == 0x01)
+		return "RL-CF900 (3.5x5)";
+	else if (brand == 0x7a && type == 0x02)
+		return "RK-CF800/4R (4x6)";
+	else if (brand == 0x7a && type == 0x04)
+		return "R2L-CF460/5R (5x7)";
+	else if (brand == 0x7a && type == 0x0f)
+		return "R68-CF400/6R (6x8)";
 	else
 		return "Unknown";
 
 // Also CK-D715, CK-D718, CK-D720, CK-D723 (4x6,5x8,6x8,6x9) for D70-S model
 //      CK-D746-U for D70-U model
 //      CK-D820 (6x8) for D80-S model
-//      CK-D868 (6x8) for D80 (non-S)
-// D90 can use _all_ of htese types except for the -U!
+// D90 can use _all_ of these types except for the -U!
 
 }
 
@@ -738,6 +769,22 @@ static void mitsu70x_attach(void *vctx, struct libusb_device_handle *dev,
 #else
 	WARNING("Dynamic library support not enabled, using internal fallback code\n");
 #endif
+
+	struct mitsu70x_printerstatus_resp resp;
+	int ret;
+
+	ret = mitsu70x_get_printerstatus(ctx, &resp);
+	if (ret) {
+		ERROR("Unable to get printer status! (%d)\n", ret);
+		return;
+	}
+
+	if (ctx->type == P_MITSU_D70X &&
+	    resp.upper.mecha_status[0] != MECHA_STATUS_INIT &&
+		resp.upper.capacity != 0xffff)
+		ctx->num_decks = 2;
+	else
+		ctx->num_decks = 1;
 }
 
 static void mitsu70x_teardown(void *vctx) {
@@ -893,16 +940,16 @@ repeat:
 		}
 	} else if (ctx->type == P_FUJI_ASK300) {
 		ctx->laminatefname = CORRTABLE_PATH "/ASK300M2.raw"; // Same as D70
-		ctx->lutfname = CORRTABLE_PATH "/CPD70L01.lut";  // XXX guess, driver did not come with external LUT!
+//		ctx->lutfname = CORRTABLE_PATH "/CPD70L01.lut";  // XXX guess, driver did not come with external LUT!
 		if (mhdr.speed == 3 || mhdr.speed == 4) {
 			mhdr.speed = 3; /* Super Fine */
 			ctx->cpcfname = CORRTABLE_PATH "/ASK300T3.cpc";
 		} else {
 			ctx->cpcfname = CORRTABLE_PATH "/ASK300T1.cpc";
 		}
-		if (mhdr.hdr[3] != 0x01) {
+		if (mhdr.hdr[3] != 0x80) {
 			WARNING("Print job has wrong submodel specifier (%x)\n", mhdr.hdr[3]);
-			mhdr.hdr[3] = 0x01;
+			mhdr.hdr[3] = 0x80;
 		}
 	}
 	if (!mhdr.use_lut)
@@ -1172,7 +1219,7 @@ static int mitsu70x_get_jobs(struct mitsu70x_ctx *ctx, struct mitsu70x_jobs *res
 }
 #endif
 
-static int mitsu70x_get_memorystatus(struct mitsu70x_ctx *ctx, struct mitsu70x_memorystatus_resp *resp)
+static int mitsu70x_get_memorystatus(struct mitsu70x_ctx *ctx, uint8_t mcut, struct mitsu70x_memorystatus_resp *resp)
 {
 	uint8_t cmdbuf[CMDBUF_LEN];
 
@@ -1188,7 +1235,17 @@ static int mitsu70x_get_memorystatus(struct mitsu70x_ctx *ctx, struct mitsu70x_m
 	cmdbuf[3] = 0x00;
 	tmp = cpu_to_be16(ctx->cols);
 	memcpy(cmdbuf + 4, &tmp, 2);
-	tmp = cpu_to_be16(ctx->rows);
+
+	/* We have to lie about print sizes in 4x6*2 multicut modes */
+	tmp = ctx->rows;
+	if (tmp == 2730 && mcut == 1) {
+		if (ctx->type == P_MITSU_D70X ||
+		    ctx->type == P_FUJI_ASK300) {
+			tmp = 2422;
+		}
+	}
+
+	tmp = cpu_to_be16(tmp);
 	memcpy(cmdbuf + 6, &tmp, 2);
 	cmdbuf[8] = ctx->matte ? 0x80 : 0x00;
 	cmdbuf[9] = 0x00;
@@ -1271,9 +1328,9 @@ static int mitsu70x_set_sleeptime(struct mitsu70x_ctx *ctx, uint8_t time)
 	uint8_t cmdbuf[4];
 	int ret;
 
-	/* 10 minutes max, according to all docs. */
-	if (time > 10)
-		time = 10;
+	/* 60 minutes max, according to all docs. */
+	if (time > 60)
+		time = 60;
 
 	/* Send Parameter.. */
 	memset(cmdbuf, 0, 4);
@@ -1342,23 +1399,40 @@ static int mitsu70x_set_printermode(struct mitsu70x_ctx *ctx, uint8_t enabled)
 	return 0;
 }
 #endif
-static int mitsu70x_wakeup(struct mitsu70x_ctx *ctx)
+static int mitsu70x_wakeup(struct mitsu70x_ctx *ctx, int wait)
 {
 	int ret;
 	uint8_t buf[512];
+	struct mitsu70x_jobstatus jobstatus;
 
-	memset(buf, 0, sizeof(buf));
-	buf[0] = 0x1b;
-	buf[1] = 0x45;
-	buf[2] = 0x57; // XXX also, 0x53, 0x54 seen.
-	buf[3] = 0x55;
-
-	INFO("Waking up printer...\n");
-	if ((ret = send_data(ctx->dev, ctx->endp_down,
-			     buf, sizeof(buf))))
+top:
+	/* Query job status for jobid 0 (global) */
+	ret = mitsu70x_get_jobstatus(ctx, &jobstatus, 0x0000);
+	if (ret)
 		return CUPS_BACKEND_FAILED;
 
-	return 0;
+	/* Trigger a wakeup if necessary */
+	if (jobstatus.power) {
+		INFO("Waking up printer...\n");
+
+		memset(buf, 0, sizeof(buf));
+		buf[0] = 0x1b;
+		buf[1] = 0x45;
+		buf[2] = 0x57; // XXX also, 0x53, 0x54 seen.
+		buf[3] = 0x55;
+
+		if ((ret = send_data(ctx->dev, ctx->endp_down,
+				     buf, sizeof(buf))))
+			return CUPS_BACKEND_FAILED;
+
+		if (wait) {
+			sleep(1);
+			goto top;
+		}
+	}
+
+
+	return CUPS_BACKEND_OK;
 }
 
 static int d70_library_callback(void *context, void *buffer, uint32_t len)
@@ -1391,6 +1465,7 @@ static int mitsu70x_main_loop(void *vctx, int copies)
 	struct mitsu70x_printerstatus_resp resp;
 	struct mitsu70x_hdr *hdr;
 	uint8_t last_status[4] = {0xff, 0xff, 0xff, 0xff};
+	int statusdump = 0;
 
 	int ret;
 
@@ -1401,22 +1476,18 @@ static int mitsu70x_main_loop(void *vctx, int copies)
 
 	INFO("Waiting for printer idle...\n");
 
+	/* Ensure printer is awake */
+	ret = mitsu70x_wakeup(ctx, 1);
+	if (ret)
+		return CUPS_BACKEND_FAILED;
+
 top:
+
 	/* Query job status for jobid 0 (global) */
 	ret = mitsu70x_get_jobstatus(ctx, &jobstatus, 0x0000);
 	if (ret)
 		return CUPS_BACKEND_FAILED;
 
-	/* Make sure we're awake! */
-	if (jobstatus.power) {
-		ret = mitsu70x_wakeup(ctx);
-		if (ret)
-			return CUPS_BACKEND_FAILED;
-
-		sleep(1);
-		goto top;
-	}
-
 	/* Make sure temperature is sane */
 	if (jobstatus.temperature == TEMPERATURE_COOLING) {
 		INFO("Printer cooling down...\n");
@@ -1436,21 +1507,16 @@ top:
 		return CUPS_BACKEND_STOP;
 	}
 
-	if (ctx->num_decks)
+	if (statusdump)
 		goto skip_status;
+	statusdump = 1;
 
 	/* Tell CUPS about the consumables we report */
 	ret = mitsu70x_get_printerstatus(ctx, &resp);
 	if (ret)
 		return CUPS_BACKEND_FAILED;
 
-	if (resp.upper.mecha_status[0] != MECHA_STATUS_INIT)
-		ctx->num_decks = 2;
-	else
-		ctx->num_decks = 1;
-
-	if (ctx->type == P_MITSU_D70X &&
-	    ctx->num_decks == 2) {
+	if (ctx->num_decks == 2) {
 		ATTR("marker-colors=#00FFFF#FF00FF#FFFF00,#00FFFF#FF00FF#FFFF00\n");
 		ATTR("marker-high-levels=100,100\n");
 		ATTR("marker-low-levels=10,10\n");
@@ -1469,11 +1535,34 @@ top:
 
 	/* FW sanity checking */
 	if (ctx->type == P_KODAK_305) {
-		if (be16_to_cpu(resp.vers[0].checksum) != EK305_0104_M_CSUM)
-			WARNING("Printer FW out of date. Highly recommend upgrading EK305 to v1.04!\n");
+		/* Known versions:
+		   v1.02: M 316E81 1433   (Add Ultrafine and matte support)
+		   v1.04: M 316F83 2878   (Add 2x6 strip and support "Triton" media)
+		*/
+		if (strncmp(resp.vers[0].ver, "316F83", 6) < 0)
+			WARNING("Printer FW out of date. Highly recommend upgrading EK305 to v1.04 or newer!\n");
+	} else if (ctx->type == P_MITSU_K60) {
+		/* Known versions:
+		   v1.05: M 316M31 148C   (Add HG media support)
+		*/
+		if (strncmp(resp.vers[0].ver, "316M31", 6) < 0)
+			WARNING("Printer FW out of date. Highly recommend upgrading K60 to v1.05 or newer!\n");
 	} else if (ctx->type == P_MITSU_D70X) {
-		if (be16_to_cpu(resp.vers[0].checksum) != MD70X_0112_M_CSUM)
-			WARNING("Printer FW out of date. Highly recommend upgrading D70/D707 to v1.12!\n");
+		/* Known versions:
+		   v1.10: M 316V11 064D   (Add ultrafine mode, 6x6 support, 2x6 strip, and more?)
+		   v1.12: M 316W11 9FC3   (??)
+		   v1.13:                 (??)
+		*/
+		if (strncmp(resp.vers[0].ver, "316W11", 6) < 0)
+			WARNING("Printer FW out of date. Highly recommend upgrading D70/D707 to v1.12 or newer!\n");
+	} else if (ctx->type == P_FUJI_ASK300) {
+		/* Known versions:
+		   v?.??: M 316A21 7998   (ancient. no matte or ultrafine)
+		   v?.??: M 316H21 F8EB
+		   v4.20a: M 316J21 4431  (Add 2x6 strip support)
+		*/
+		if (strncmp(resp.vers[0].ver, "316J21", 6) < 0)
+			WARNING("Printer FW out of date. Highly recommend upgrading ASK300 to v4.20a or newer!\n");
 	}
 
 skip_status:
@@ -1482,7 +1571,7 @@ skip_status:
 		struct mitsu70x_memorystatus_resp memory;
 		INFO("Checking Memory availability\n");
 
-		ret = mitsu70x_get_memorystatus(ctx, &memory);
+		ret = mitsu70x_get_memorystatus(ctx, hdr->multicut, &memory);
 		if (ret)
 			return CUPS_BACKEND_FAILED;
 
@@ -1541,7 +1630,7 @@ skip_status:
 		DEBUG("Rewind Inhibit? %02x %02x\n", hdr->rewind[0], hdr->rewind[1]);
 	}
 
-	/* Any other fixups? */
+	/* K60 and EK305 need the mcut type 1 specified for 4x6 prints! */
 	if ((ctx->type == P_MITSU_K60 || ctx->type == P_KODAK_305) &&
 	    ctx->cols == 0x0748 &&
 	    ctx->rows == 0x04c2 && !hdr->multicut) {
@@ -1594,19 +1683,26 @@ skip_status:
 
 		donor_l = be16_to_cpu(resp.lower.remain) * 100 / be16_to_cpu(resp.lower.capacity);
 
-		if (ctx->type == P_MITSU_D70X &&
-		    ctx->num_decks == 2) {
+		if (ctx->num_decks == 2) {
 			donor_u = be16_to_cpu(resp.upper.remain) * 100 / be16_to_cpu(resp.upper.capacity);
 			if (donor_l != ctx->last_donor_l ||
 			    donor_u != ctx->last_donor_u) {
 				ctx->last_donor_l = donor_l;
 				ctx->last_donor_u = donor_u;
 				ATTR("marker-levels=%d,%d\n", donor_l, donor_u);
+				ATTR("marker-message='\"%d native prints remaining on %s media\"','\"%d native prints remaining on %s media\"'\n",
+				     be16_to_cpu(resp.lower.remain),
+				     mitsu70x_media_types(resp.lower.media_brand, resp.lower.media_type),
+				     be16_to_cpu(resp.upper.remain),
+				     mitsu70x_media_types(resp.upper.media_brand, resp.upper.media_type));
 			}
 		} else {
 			if (donor_l != ctx->last_donor_l) {
 				ctx->last_donor_l = donor_l;
 				ATTR("marker-levels=%d\n", donor_l);
+				ATTR("marker-message=\"%d native prints remaining on %s media\"\n",
+				     be16_to_cpu(resp.lower.remain),
+				     mitsu70x_media_types(resp.lower.media_brand, resp.lower.media_type));
 			}
 		}
 
@@ -1653,7 +1749,7 @@ skip_status:
 			break;
 		}
 
-		if (fast_return) {
+		if (fast_return && copies <= 1) { /* Copies generated by backend! */
 			INFO("Fast return mode enabled.\n");
 			break;
 		}
@@ -1675,7 +1771,8 @@ skip_status:
 	return CUPS_BACKEND_OK;
 }
 
-static void mitsu70x_dump_printerstatus(struct mitsu70x_printerstatus_resp *resp)
+static void mitsu70x_dump_printerstatus(struct mitsu70x_ctx *ctx,
+					struct mitsu70x_printerstatus_resp *resp)
 {
 	uint32_t i;
 
@@ -1708,83 +1805,79 @@ static void mitsu70x_dump_printerstatus(struct mitsu70x_printerstatus_resp *resp
 	}
 	INFO("Standby Timeout: %d minutes\n", resp->sleeptime);
 	INFO("iSerial Reporting: %s\n", resp->iserial ? "No" : "Yes" );
+	INFO("Power Status: %s\n", resp->power ? "Sleeping" : "Awake");
 
-	INFO("Lower Mechanical Status: %s\n",
-	     mitsu70x_mechastatus(resp->lower.mecha_status));
 	if (resp->lower.error_status[0]) {
 		INFO("Lower Error Status: %s/%s -> %s\n",
 		     mitsu70x_errorclass(resp->lower.error_status),
 		     mitsu70x_errors(resp->lower.error_status),
 		     mitsu70x_errorrecovery(resp->lower.error_status));
 	}
-	INFO("Lower Media type:  %s (%02x/%02x)\n",
+	INFO("Lower Temperature: %s\n", mitsu70x_temperatures(resp->lower.temperature));
+	INFO("Lower Mechanical Status: %s\n",
+	     mitsu70x_mechastatus(resp->lower.mecha_status));
+	INFO("Lower Media Type:  %s (%02x/%02x)\n",
 	     mitsu70x_media_types(resp->lower.media_brand, resp->lower.media_type),
 	     resp->lower.media_brand,
 	     resp->lower.media_type);
-	INFO("Lower Prints remaining:  %03d/%03d\n",
+	INFO("Lower Prints Remaining:  %03d/%03d\n",
 	     be16_to_cpu(resp->lower.remain),
 	     be16_to_cpu(resp->lower.capacity));
-
 	i = packed_bcd_to_uint32((char*)resp->lower.lifetime_prints, 4);
 	if (i)
 		i-= 10;
-	INFO("Lower Lifetime prints:  %u\n", i);
+	INFO("Lower Lifetime Prints:  %u\n", i);
 
-	if (resp->upper.mecha_status[0] != MECHA_STATUS_INIT) {
-		INFO("Upper Mechanical Status: %s\n",
-		     mitsu70x_mechastatus(resp->upper.mecha_status));
+	if (ctx->num_decks == 2) {
 		if (resp->upper.error_status[0]) {
 			INFO("Upper Error Status: %s/%s -> %s\n",
 			     mitsu70x_errorclass(resp->upper.error_status),
 			     mitsu70x_errors(resp->upper.error_status),
 			     mitsu70x_errorrecovery(resp->upper.error_status));
 		}
-		INFO("Upper Media type:  %s (%02x/%02x)\n",
+		INFO("Upper Temperature: %s\n", mitsu70x_temperatures(resp->upper.temperature));
+		INFO("Upper Mechanical Status: %s\n",
+		     mitsu70x_mechastatus(resp->upper.mecha_status));
+		INFO("Upper Media Type:  %s (%02x/%02x)\n",
 		     mitsu70x_media_types(resp->upper.media_brand, resp->upper.media_type),
 		     resp->upper.media_brand,
 		     resp->upper.media_type);
-		INFO("Upper Prints remaining:  %03d/%03d\n",
+		INFO("Upper Prints Remaining:  %03d/%03d\n",
 		     be16_to_cpu(resp->upper.remain),
 		     be16_to_cpu(resp->upper.capacity));
 
 		i = packed_bcd_to_uint32((char*)resp->upper.lifetime_prints, 4);
 		if (i)
 			i-= 10;
-		INFO("Upper Lifetime prints:  %u\n", i);
+		INFO("Upper Lifetime Prints:  %u\n", i);
 	}
 }
 
-static int mitsu70x_query_status(struct mitsu70x_ctx *ctx)
+static int mitsu70x_query_jobs(struct mitsu70x_ctx *ctx)
 {
-	struct mitsu70x_printerstatus_resp resp;
 #if 0
 	struct mitsu70x_jobs jobs;
 #endif
 	struct mitsu70x_jobstatus jobstatus;
-
 	int ret;
 
-top:
 	ret = mitsu70x_get_jobstatus(ctx, &jobstatus, 0x0000);
 	if (ret)
-		goto done;
-
-	/* Make sure we're awake! */
-	if (jobstatus.power) {
-		ret = mitsu70x_wakeup(ctx);
-		if (ret)
-			return CUPS_BACKEND_FAILED;
-
-		sleep(1);
-		goto top;
-	}
-
-	ret = mitsu70x_get_printerstatus(ctx, &resp);
-	if (!ret)
-		mitsu70x_dump_printerstatus(&resp);
+		return CUPS_BACKEND_FAILED;
 
 	INFO("JOB00 ID     : %06u\n", jobstatus.jobid);
 	INFO("JOB00 status : %s\n", mitsu70x_jobstatuses(jobstatus.job_status));
+	INFO("Power Status: %s\n", jobstatus.power ? "Sleeping" : "Awake");
+	INFO("Mechanical Status: %s\n",
+	     mitsu70x_mechastatus(jobstatus.mecha_status));
+	if (jobstatus.error_status[0]) {
+		INFO("%s/%s -> %s\n",
+		     mitsu70x_errorclass(jobstatus.error_status),
+		     mitsu70x_errors(jobstatus.error_status),
+		     mitsu70x_errorrecovery(jobstatus.error_status));
+	}
+	INFO("Temperature: %s\n", mitsu70x_temperatures(jobstatus.temperature));
+	// memory status?
 
 #if 0
 	ret = mitsu70x_get_jobs(ctx, &jobs);
@@ -1797,9 +1890,21 @@ top:
 			INFO("JOB%02d status : %s\n", i, mitsu70x_jobstatuses(jobs.jobs[i].status));
 		}
 	}
-#endif
 
 done:
+#endif
+	return CUPS_BACKEND_OK;
+}
+
+static int mitsu70x_query_status(struct mitsu70x_ctx *ctx)
+{
+	struct mitsu70x_printerstatus_resp resp;
+	int ret;
+
+	ret = mitsu70x_get_printerstatus(ctx, &resp);
+	if (!ret)
+		mitsu70x_dump_printerstatus(ctx, &resp);
+
 	return ret;
 }
 
@@ -1830,7 +1935,10 @@ static int mitsu70x_query_serno(struct libusb_device_handle *dev, uint8_t endp_u
 
 static void mitsu70x_cmdline(void)
 {
-	DEBUG("\t\t[ -s ]           # Query status\n");
+	DEBUG("\t\t[ -s ]           # Query printer status\n");
+	DEBUG("\t\t[ -j ]           # Query job status\n");
+	DEBUG("\t\t[ -w ]           # Wake up printer\n");
+	DEBUG("\t\t[ -W ]           # Wake up printer and wait\n");
 	DEBUG("\t\t[ -f ]           # Use fast return mode\n");
 	DEBUG("\t\t[ -k num ]       # Set standby time (1-60 minutes, 0 disables)\n");
 	DEBUG("\t\t[ -x num ]       # Set USB iSerialNumber Reporting (1 on, 0 off)\n");
@@ -1844,15 +1952,24 @@ static int mitsu70x_cmdline_arg(void *vctx, int argc, char **argv)
 	if (!ctx)
 		return -1;
 
-	while ((i = getopt(argc, argv, GETOPT_LIST_GLOBAL "sk:X:x:")) >= 0) {
+	while ((i = getopt(argc, argv, GETOPT_LIST_GLOBAL "jk:swWX:x:")) >= 0) {
 		switch(i) {
 		GETOPT_PROCESS_GLOBAL
+		case 'j':
+			j = mitsu70x_query_jobs(ctx);
+			break;
 		case 'k':
 			j = mitsu70x_set_sleeptime(ctx, atoi(optarg));
 			break;
 		case 's':
 			j = mitsu70x_query_status(ctx);
 			break;
+		case 'w':
+			j = mitsu70x_wakeup(ctx, 0);
+			break;
+		case 'W':
+			j = mitsu70x_wakeup(ctx, 1);
+			break;
 		case 'x':
 			j = mitsu70x_set_iserial(ctx, atoi(optarg));
 			break;
@@ -1873,7 +1990,7 @@ static int mitsu70x_cmdline_arg(void *vctx, int argc, char **argv)
 /* Exported */
 struct dyesub_backend mitsu70x_backend = {
 	.name = "Mitsubishi CP-D70/D707/K60/D80",
-	.version = "0.61",
+	.version = "0.74",
 	.uri_prefix = "mitsu70x",
 	.cmdline_usage = mitsu70x_cmdline,
 	.cmdline_arg = mitsu70x_cmdline_arg,
@@ -1884,13 +2001,13 @@ struct dyesub_backend mitsu70x_backend = {
 	.main_loop = mitsu70x_main_loop,
 	.query_serno = mitsu70x_query_serno,
 	.devices = {
-		{ USB_VID_MITSU, USB_PID_MITSU_D70X, P_MITSU_D70X, ""},
-		{ USB_VID_MITSU, USB_PID_MITSU_K60, P_MITSU_K60, ""},
-		{ USB_VID_MITSU, USB_PID_MITSU_D80, P_MITSU_D80, ""},
-//		{ USB_VID_MITSU, USB_PID_MITSU_D90, P_MITSU_D90, ""},
-		{ USB_VID_KODAK, USB_PID_KODAK305, P_KODAK_305, ""},
-//	{ USB_VID_FUJIFILM, USB_PID_FUJI_ASK300, P_FUJI_ASK300, ""},
-	{ 0, 0, 0, ""}
+		{ USB_VID_MITSU, USB_PID_MITSU_D70X, P_MITSU_D70X, NULL},
+		{ USB_VID_MITSU, USB_PID_MITSU_K60, P_MITSU_K60, NULL},
+		{ USB_VID_MITSU, USB_PID_MITSU_D80, P_MITSU_D80, NULL},
+//		{ USB_VID_MITSU, USB_PID_MITSU_D90, P_MITSU_D90, NULL},
+		{ USB_VID_KODAK, USB_PID_KODAK305, P_KODAK_305, NULL},
+		{ USB_VID_FUJIFILM, USB_PID_FUJI_ASK300, P_FUJI_ASK300, NULL},
+	{ 0, 0, 0, NULL}
 	}
 };
 
@@ -1913,23 +2030,23 @@ struct dyesub_backend mitsu70x_backend = {
    1b 5a 54 PP JJ JJ RR RR  00 00 00 00 00 00 00 00
    XX XX YY YY QQ QQ ZZ ZZ  SS 00 00 00 00 00 00 00
    UU 00 00 00 00 00 00 00  LL TT 00 00 00 00 00 00
-   RR 00 00 00 00 00 00 00  00 00 00 00 00 00 00 00
+   MM 00 00 00 00 00 00 00  00 00 00 00 00 00 00 00
 
    (padded by NULLs to a 512-byte boundary)
 
-   PP    == 0x01 on D70x/D80/ASK300, 0x00 on K60, 0x90 on K305
+   PP    == 0x01 on D70x/D80, 0x00 on K60, 0x90 on K305, 0x80 on ASK300
    JJ JJ == Job ID, can leave at 00 00
    XX XX == columns
    YY YY == rows
    QQ QQ == lamination columns (equal to XX XX)
    ZZ ZZ == lamination rows (YY YY + 12 on D70x/D80/ASK300, YY YY on others)
-   RR RR == "rewind inhibit", 01 01 enabled, normally 00 00 (All but D70x)
+   RR RR == "rewind inhibit", 01 01 enabled, normally 00 00 (All but D70x/A300)
    SS    == Print mode: 00 = Fine, 03 = SuperFine (D70x/D80 only), 04 = UltraFine
             (Matte requires Superfine or Ultrafine)
    UU    == 00 = Auto, 01 = Lower Deck (required for !D70x), 02 = Upper Deck
    LL    == lamination enable, 00 == on, 01 == off
-   TT    == lamination mode: 00 glossy, 02 matte.
-   RR    == 00 (normal), 01 = (Double-cut 4x6), 05 = (double-cut 2x6)
+   TT    == lamination mode: 00 glossy, 02 matte
+   MM    == 00 (normal), 01 = (Double-cut 4x6), 05 = (double-cut 2x6)
 
    Data planes:
    16-bit data, rounded up to 512-byte block (XX * YY * 2 bytes)
