@@ -20,11 +20,12 @@
  *   for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *   along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
  *          [http://www.gnu.org/licenses/gpl-2.0.html]
  *
+ *   SPDX-License-Identifier: GPL-2.0+
+ *
  */
 
 #include <stdio.h>
@@ -395,7 +396,7 @@ struct shinkos1245_resp_matte {
 	uint8_t code;
 	uint8_t mode;
 	 int8_t level;
-	uint8_t reserved[3];
+	uint8_t reserved[4];
 } __attribute__((packed));
 
 #define MATTE_MODE_MATTE 0x00
@@ -497,6 +498,7 @@ static int shinkos1245_get_media(struct shinkos1245_ctx *ctx)
 
 	shinkos1245_fill_hdr(&cmd.hdr);
 	memset(cmd.pad, 0, sizeof(cmd.pad));
+	ctx->media_8x12 = 0;
 	for (i = 1 ; i <= 3 ; i++) {
 		cmd.cmd[0] = 0x0a | (i << 4);
 
@@ -512,20 +514,21 @@ static int shinkos1245_get_media(struct shinkos1245_ctx *ctx)
 			return -99;
 		}
 
-		if (resp.count > NUM_MEDIAS)
-			resp.count = NUM_MEDIAS;
-
 		/* Store media info */
-		for (j = 0; j < resp.count ; j++) {
+		for (j = 0; j < NUM_MEDIAS && ctx->num_medias < resp.count ; j++) {
 			ctx->medias[ctx->num_medias].code = resp.data[j].code;
 			ctx->medias[ctx->num_medias].columns = be16_to_cpu(resp.data[j].columns);
 			ctx->medias[ctx->num_medias].rows = be16_to_cpu(resp.data[j].rows);
 			ctx->medias[ctx->num_medias].type = resp.data[j].type;
 			ctx->medias[ctx->num_medias].print_type = resp.data[j].print_type;
 			ctx->num_medias++;
+
+			if (ctx->medias[i].rows >= 3636)
+				ctx->media_8x12 = 1;
 		}
 
-		if (resp.count < 5)
+		/* Once we've parsed them all.. we're done */
+		if (ctx->num_medias == resp.count)
 			break;
 	}
 	return ret;
@@ -566,7 +569,7 @@ static int shinkos1245_set_printerid(struct shinkos1245_ctx *ctx,
 
 	for (i = 0 ; i < (int)sizeof(cmd.data) ; i++) {
 		if (*id)
-			cmd.data[i] = (uint8_t) *id;
+			cmd.data[i] = (uint8_t) *id++;
 		else
 			cmd.data[i] = ' ';
 	}
@@ -933,11 +936,12 @@ static void shinkos1245_dump_status(struct shinkos1245_resp_status *sts)
 	INFO("Tone Curve Status: %s\n", detail);
 }
 
-static void shinkos1245_dump_media(struct shinkos1245_mediadesc *medias,
+static void shinkos1245_dump_media(struct shinkos1245_mediadesc *medias, int media_8x12,
 				   int count)
 {
 	int i;
 
+	INFO("Loaded media type: %s\n", media_8x12 ? "8x12" : "8x10");
 	INFO("Supported print sizes: %d\n", count);
 
 	for (i = 0 ; i < count ; i++) {
@@ -945,7 +949,7 @@ static void shinkos1245_dump_media(struct shinkos1245_mediadesc *medias,
 		     medias[i].print_type,
 		     medias[i].columns,
 		     medias[i].rows,
-		     medias[i].code, medias[i].type);
+		     medias[i].type, medias[i].print_type);
 	}
 }
 
@@ -1214,7 +1218,7 @@ int shinkos1245_cmdline_arg(void *vctx, int argc, char **argv)
 		case 'm':
 			j = shinkos1245_get_media(ctx);
 			if (!j)
-				shinkos1245_dump_media(ctx->medias, ctx->num_medias);
+				shinkos1245_dump_media(ctx->medias, ctx->media_8x12, ctx->num_medias);
 			break;
 		case 'R':
 			j = shinkos1245_reset(ctx);
@@ -1404,9 +1408,6 @@ static int shinkos1245_main_loop(void *vctx, int copies) {
 	}
 	/* Make sure print size is supported */
 	for (i = 0 ; i < ctx->num_medias ; i++) {
-		if (ctx->medias[i].rows >= 3636)
-			ctx->media_8x12 = 1;
-
 		if (ctx->hdr.media == ctx->medias[i].code &&
 		    ctx->hdr.method == ctx->medias[i].print_type &&
 		    ctx->hdr.rows == ctx->medias[i].rows &&
@@ -1454,10 +1455,11 @@ top:
 
 	/* Work out the remaining media percentage */
 	{
-		int remain = ctx->media_8x12 ? 230 : 280;
+		int total = ctx->media_8x12 ? 230 : 280;
+		int remain = total - be32_to_cpu(status1.counters.media);
 
-		remain = (remain - be32_to_cpu(status1.counters.media)) * 100 / remain;
-		ATTR("marker-levels=%d\n", remain);
+		ATTR("marker-levels=%d\n", remain * 100 / total);
+		ATTR("marker-message=\"%d prints remaining on ribbon\"\n", remain);
 	}
 
 	last_state = state;
@@ -1515,7 +1517,7 @@ top:
 				if (i < 0)
 					goto printer_error;
 				if (i > 0) {
-					INFO("Can't set matte intensity when printing in progres...\n");
+					INFO("Can't set matte intensity when printing in progress...\n");
 					state = S_IDLE;
 					sleep(1);
 					break;
@@ -1640,7 +1642,7 @@ static int shinkos1245_query_serno(struct libusb_device_handle *dev, uint8_t end
 
 struct dyesub_backend shinkos1245_backend = {
 	.name = "Shinko/Sinfonia CHC-S1245",
-	.version = "0.13WIP",
+	.version = "0.18",
 	.uri_prefix = "shinkos1245",
 	.cmdline_usage = shinkos1245_cmdline,
 	.cmdline_arg = shinkos1245_cmdline_arg,
@@ -1651,8 +1653,8 @@ struct dyesub_backend shinkos1245_backend = {
 	.main_loop = shinkos1245_main_loop,
 	.query_serno = shinkos1245_query_serno,
 	.devices = {
-	{ USB_VID_SHINKO, USB_PID_SHINKO_S1245, P_SHINKO_S1245, ""},
-	{ 0, 0, 0, ""}
+	{ USB_VID_SHINKO, USB_PID_SHINKO_S1245, P_SHINKO_S1245, NULL},
+	{ 0, 0, 0, NULL}
 	}
 };
 
