@@ -20,11 +20,12 @@
  *   for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *   along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
  *          [http://www.gnu.org/licenses/gpl-2.0.html]
  *
+ *   SPDX-License-Identifier: GPL-2.0+
+ *
  */
 
 #include <stdio.h>
@@ -178,7 +179,7 @@ static char *cmd_names(uint16_t v) {
 	default:
 		return "Unknown Command";
 	}
-};
+}
 
 struct s6245_print_cmd {
 	struct s6245_cmd_hdr hdr;
@@ -1892,10 +1893,12 @@ static int shinkos6245_query_serno(struct libusb_device_handle *dev, uint8_t end
 /* Exported */
 #define USB_VID_SHINKO       0x10CE
 #define USB_PID_SHINKO_S6245 0x001D
+#define USB_VID_HITI         0x0D16
+#define USB_PID_HITI_P910L   0x000E
 
 struct dyesub_backend shinkos6245_backend = {
 	.name = "Shinko/Sinfonia CHC-S6245",
-	.version = "0.07WIP",
+	.version = "0.08WIP",
 	.uri_prefix = "shinkos6245",
 	.cmdline_usage = shinkos6245_cmdline,
 	.cmdline_arg = shinkos6245_cmdline_arg,
@@ -1906,8 +1909,9 @@ struct dyesub_backend shinkos6245_backend = {
 	.main_loop = shinkos6245_main_loop,
 	.query_serno = shinkos6245_query_serno,
 	.devices = {
-	{ USB_VID_SHINKO, USB_PID_SHINKO_S6245, P_SHINKO_S6245, ""},
-	{ 0, 0, 0, ""}
+	{ USB_VID_SHINKO, USB_PID_SHINKO_S6245, P_SHINKO_S6245, NULL},
+	{ USB_VID_HITI, USB_PID_HITI_P910L, P_SHINKO_S6245, NULL},
+	{ 0, 0, 0, NULL}
 	}
 };
 
