@@ -14,8 +14,7 @@
  *   for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *   along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
  * Contents:
  *
@@ -1124,6 +1123,7 @@ main(int  argc,				/* I - Number of command-line arguments */
   (void) gettimeofday(&t1, NULL);
   stp_init();
   version_id = stp_get_version();
+  default_settings = stp_vars_create();
 
  /*
   * Check for valid arguments...
@@ -1289,7 +1289,6 @@ main(int  argc,				/* I - Number of command-line arguments */
   if (! suppress_messages)
     fprintf(stderr, "DEBUG: Gutenprint: Using fd %d\n", fd);
 
-  default_settings = stp_vars_create_copy(stp_printer_get_defaults(printer));
   stp_set_printer_defaults(default_settings, printer);
 #ifdef ENABLE_CUPS_LOAD_SAVE_OPTIONS
   if (load_file_name)
