@@ -16,8 +16,7 @@
  *   for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *   along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 /*
@@ -969,6 +968,7 @@ stp_family_register(stp_list_t *family)
 
   if (family)
     {
+      int duplicate_printers = 0;
       printer_item = stp_list_get_start(family);
 
       while(printer_item)
@@ -977,10 +977,18 @@ stp_family_register(stp_list_t *family)
 	  if (!stp_list_get_item_by_name(printer_list, printer->driver))
 	    stp_list_item_create(printer_list, NULL, printer);
 	  else
-	    stp_erprintf("Duplicate printer entry `%s' (%s)\n",
-			 printer->driver, printer->long_name);
+	    {
+	      stp_erprintf("Duplicate printer entry `%s' (%s)\n",
+			   printer->driver, printer->long_name);
+	      duplicate_printers++;
+	    }
 	  printer_item = stp_list_item_next(printer_item);
 	}
+      if (duplicate_printers)
+	{
+	  stp_erprintf("FATAL Duplicate printers in printer list.  Aborting!\n");
+	  stp_abort();
+	}
     }
 
   return 0;
